local L = LibStub("AceLocale-3.0"):GetLocale("Procodile", false)
local media = LibStub("LibSharedMedia-3.0")

Procodile.defaults = {
        profile = {
				-- Cooldown bars
                cooldowns = true,
                movablecooldowns = true,
                barfontsize=11,
                barwidth=130,
                barheight=14,
                barfont=nil,
                bartexture="Aluminium",
                reversecooldowns=false,

				-- Aura bars
                aurabars = false,
                movableaurabars = true,
                announceauras = true,
                aurabarfontsize=11,
                aurabarwidth=130,
                aurabarheight=14,
                aurabarfont=nil,
                aurabartexture="Aluminium",
                reverseaurabars=false,

				-- General options
                combat = true,
                showspells = false,
                verbose = false,
                mmbutton=true,
                icon = {},
                announce = true,
                announceflash=true,
                announceshake=false,
                
                -- internal
                enabled = false,
				tracked = {},
				dormant = {},
                sinkOptions = {
                	sink20OutputSink = "Default",
				},
        }
}

Procodile.options = {
        type="group",
		name="Procodile",
        childGroups="tab",
        args={
        		d = {
        			type="description",
					name=L["PROCODILE_DESC"],
        		},
        		
-- Estimated cooldown bars
        		bars = {
	        			type="group",
                        name=L["BARS_NAME"],
                        order=1,
                        args = {
                        	info = {
                        		type="description",
		                        name=L["BARS_DESC"],
		                        order=0,
                        	},
                            cooldowns = {
                                    type="toggle",
                                    name=L["COOLDOWNS_NAME"],
                                    desc=L["COOLDOWNS_DESC"],
                                    order=1,
                                    get=function() return Procodile.db.profile.cooldowns end,
                                    set=function() 
                                    		Procodile.db.profile.cooldowns = not Procodile.db.profile.cooldowns
                                   			Procodile:ShowCooldowns(Procodile.db.profile.cooldowns or false)
                                    	end,
                            },
                            movablecooldowns = {
                                    type="toggle",
                                    name=L["MOVABLECOOLDOWNS_NAME"],
                                    desc=L["MOVABLECOOLDOWNS_DESC"],
                                    order=2,
                                    get=function() return Procodile.db.profile.movablecooldowns end,
                                    set=function() 
                                    		Procodile.db.profile.movablecooldowns = not Procodile.db.profile.movablecooldowns
                                    		Procodile:SetMovableCooldowns(Procodile.db.profile.movablecooldowns)
                                    	end,
                            },
                    		bartexture = {
                    			type="select",
                    			name=L["BARTEXTURE_NAME"],
                    			desc=L["BARTEXTURE_DESC"],
                    			order=4,
                    			values=media:List('statusbar'),
                    			get=function(i)
                   						for k, v in pairs(media:List('statusbar')) do
								if v == Procodile.db.profile.bartexture then
									return k
								end
							end
                    				end,
                    			set=function(i, v)
                    					Procodile.db.profile.bartexture = media:List('statusbar')[v]
                    					Procodile:ReconfigureBars()
						end,
                    		}, 
                    		barfont = {
                    			type="select",
                    			name=L["BARFONT_NAME"],
                    			desc=L["BARFONT_DESC"],
                    			order=5,
                    			values=media:List('font'),
                    			get=function()
                   						for k, v in pairs(media:List('font')) do
								if v == Procodile.db.profile.barfont then
									return k
								end
							end
                    				end,
                    			set=function(k, v)
                    					Procodile.db.profile.barfont = media:List('font')[v]
                    					Procodile:ReconfigureBars()
						end,
                    		},                        		
                    		fontsize = {
                    			type="range",
                    			name=L["FONTSIZE_NAME"],
                    			desc=L["FONTSIZE_DESC"],
                    			min=6,
                    			max=20,
                    			step=1,
                    			order=5,
                    			get=function() return Procodile.db.profile.barfontsize end,
                    			set=function(key, size)
                    					Procodile.db.profile.barfontsize = size
                    					Procodile:ReconfigureBars()
						end,
                    		},
                    		barwidth = {
                    			type="range",
                    			name=L["BARWIDTH_NAME"],
                    			desc=L["BARWIDTH_DESC"],
                    			min=40,
                    			max=300,
                    			step=1,
                    			order=10,
                    			get=function() return Procodile.db.profile.barwidth end,
                    			set=function(key, size)
                    					Procodile.db.profile.barwidth = size
                    					Procodile:ReconfigureBars()
						end,
                    		},
                    		barheight = {
                    			type="range",
                    			name=L["BARHEIGHT_NAME"],
                    			desc=L["BARHEIGHT_DESC"],
                    			min=8,
                    			max=60,
                    			step=1,
                    			order=11,
                    			get=function() return Procodile.db.profile.barheight end,
                    			set=function(key, size)
                    					Procodile.db.profile.barheight = size
							Procodile:ReconfigureBars()
                    				end,
                    		},
                            reversebars = {
                                    type="toggle",
                                    name=L["REVERSECOOLDOWNS_NAME"],
                                    desc=L["REVERSECOOLDOWNS_DESC"],
                                    order=9,
                                    get=function() return Procodile.db.profile.reversecooldowns end,
                                    set=function() 
                                    		Procodile.db.profile.reversecooldowns = not Procodile.db.profile.reversecooldowns
                                   			Procodile:ReverseCooldowns(Procodile.db.profile.reversecooldowns)
                                    	end,
                            },                           
                            bartest = {
                    			type="execute",
                    			order=-1,
                    			name=L["BARTEST_NAME"],
                    			desc=L["BARTEST_DESC"],
                    			func=function() Procodile:TestBars() end,
                    		}
                    }
        		},

-- Aura notifier bars
-- TODO: handle bar groups generically instead of this copy & paste-fest
        		aurabars = {
	        			type="group",
                        name=L["AURASECTION_NAME"],
                        order=2,
                        args = {
                        	info = {
                        		type="description",
		                        name=L["AURASECTION_DESC"],
		                        order=0,
                        	},
                            aurabars = {
                                    type="toggle",
                                    name=L["AURABARS_NAME"],
                                    desc=L["AURABARS_DESC"],
                                    order=1,
                                    get=function() return Procodile.db.profile.aurabars end,
                                    set=function() 
                                    		Procodile.db.profile.aurabars = not Procodile.db.profile.aurabars
											Procodile:ShowAurabars(Procodile.db.profile.aurabars or false)
                                    	end,
                            },
                            movableaurabars = {
                                    type="toggle",
                                    name=L["MOVABLEAURABARS_NAME"],
                                    desc=L["MOVABLEAURABARS_DESC"],
                                    order=2,
                                    get=function() return Procodile.db.profile.movableaurabars end,
                                    set=function() 
                                    		Procodile.db.profile.movableaurabars = not Procodile.db.profile.movableaurabars
                                    		Procodile:SetMovableAurabars(Procodile.db.profile.movableaurabars)
                                    	end,
                            },
                    		bartexture = {
                    			type="select",
                    			name=L["BARTEXTURE_NAME"],
                    			desc=L["BARTEXTURE_DESC"],
                    			order=4,
                    			values=media:List('statusbar'),
                    			get=function(i)
                   						for k, v in pairs(media:List('statusbar')) do
										if v == Procodile.db.profile.aurabartexture then
											return k
										end
									end
                    				end,
                    			set=function(i, v)
                    					Procodile.db.profile.aurabartexture = media:List('statusbar')[v]
                    					Procodile:ReconfigureBars()
						end,
                    		}, 
                    		barfont = {
                    			type="select",
                    			name=L["BARFONT_NAME"],
                    			desc=L["BARFONT_DESC"],
                    			order=5,
                    			values=media:List('font'),
                    			get=function()
                   						for k, v in pairs(media:List('font')) do
											if v == Procodile.db.profile.aurabarfont then
												return k
											end
										end
                    				end,
                    			set=function(k, v)
                    					Procodile.db.profile.aurabarfont = media:List('font')[v]
                    					Procodile:ReconfigureBars()
									end,
                    		},                        		
                    		fontsize = {
                    			type="range",
                    			name=L["FONTSIZE_NAME"],
                    			desc=L["FONTSIZE_DESC"],
                    			min=6,
                    			max=20,
                    			step=1,
                    			order=5,
                    			get=function() return Procodile.db.profile.aurabarfontsize end,
                    			set=function(key, size)
                    					Procodile.db.profile.aurabarfontsize = size
                    					Procodile:ReconfigureBars()
						end,
                    		},
                    		barwidth = {
                    			type="range",
                    			name=L["BARWIDTH_NAME"],
                    			desc=L["BARWIDTH_DESC"],
                    			min=40,
                    			max=300,
                    			step=1,
                    			order=10,
                    			get=function() return Procodile.db.profile.aurabarwidth end,
                    			set=function(key, size)
                    					Procodile.db.profile.aurabarwidth = size
                    					Procodile:ReconfigureBars()
						end,
                    		},
                    		barheight = {
                    			type="range",
                    			name=L["BARHEIGHT_NAME"],
                    			desc=L["BARHEIGHT_DESC"],
                    			min=8,
                    			max=60,
                    			step=1,
                    			order=11,
                    			get=function() return Procodile.db.profile.aurabarheight end,
                    			set=function(key, size)
                    					Procodile.db.profile.aurabarheight = size
										Procodile:ReconfigureBars()
                    				end,
                    		},
                            reversebars = {
                                    type="toggle",
                                    name=L["REVERSECOOLDOWNS_NAME"],
                                    desc=L["REVERSECOOLDOWNS_DESC"],
                                    order=9,
                                    get=function() return Procodile.db.profile.reverseaurabars end,
                                    set=function() 
                                    		Procodile.db.profile.reverseaurabars = not Procodile.db.profile.reverseaurabars
                                   			Procodile:ReverseAurabars(Procodile.db.profile.reverseaurabars)
                                    	end,
                            },
                            bartest = {
                    			type="execute",
                    			order=-1,
                    			name=L["BARTEST_NAME"],
                    			desc=L["BARTEST_DESC"],
                    			func=function() Procodile:TestBars() end,
                    		}
                    }
        		},
        		
        		custom = {
        			type="group",
        			name=L["CUSTOM_NAME"],
                    order=3,
        			args= {
        				info = {
        					type="description",
        					name=L["CUSTOM_DESC"],
        					order=0,
        				},
        				newcustom = {
        					type="input",
        					name=L["NEWCUSTOM_NAME"],
        					desc=L["NEWCUSTOM_DESC"],
        					order=1,
        					set=function(info, val) 
									local name, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(val)
	        						if name then
	        							Procodile:AddSpell(tonumber(val), name, icon, true)
	        						end
        						end,
        				},
        				removecustom = {
        					type="select",
        					name=L["REMOVECUSTOM_NAME"],
        					desc=L["REMOVECUSTOM_DESC"],
        					order=2,
        					values=function()
        								local customs = {}
        								for index,spell in pairs(Procodile.db.profile.tracked) do
        									if spell.custom ~= nil then
        										customs[spell.id] = spell.name
        									end
        								end
        								return customs
        							end,
        					set=function(i, v)
       								for index,spell in pairs(Procodile.db.profile.tracked) do
       									if spell.id == v then
       										table.remove(Procodile.db.profile.tracked, index)
       										break;
       									end
       								end
        						end,
        				}
        			}	
        		},
                options = {
                       type="group",
                       name=L["OPTIONS_NAME"],
                       order=4,
                       args = {
	                     	info = {
	                      		type="description",
		                        name=L["OPTIONS_DESC"],
		                        order=0,
	                      	},
							combat = {
							        type="toggle",
							        name=L["COMBATONLY_NAME"],
							        desc=L["COMBATONLY_DESC"],
							        order=1,
							        get=function() return Procodile.db.profile.combat end,
							        set=function() Procodile.db.profile.combat = not Procodile.db.profile.combat end,
							},
							showspells = {
							        type="toggle",
							        name=L["SHOWSPELLS_NAME"],
							        desc=L["SHOWSPELLS_DESC"],
							        order=2,
							        get=function() return Procodile.db.profile.showspells end,
							        set=function() Procodile.db.profile.showspells = not Procodile.db.profile.showspells end,
							},
							verbose = {
							        type="toggle",
							        name=L["VERBOSE_NAME"],
							        desc=L["VERBOSE_DESC"],
							        order=3,
							        get=function() return Procodile.db.profile.verbose end,
							        set=function() Procodile.db.profile.verbose = not Procodile.db.profile.verbose end,
							},
							mmbutton = {
							        type="toggle",
							        name=L["MINIMAP_NAME"],
							        desc=L["MINIMAP_DESC"],
							        order=4,
							        get=function() return Procodile.db.profile.mmbutton end,
							        set=function()
							        		Procodile.db.profile.mmbutton = not Procodile.db.profile.mmbutton
											Procodile:ShowMMButton(Procodile.db.profile.mmbutton)
							        	end,
							},
                            announce = {
                                    type="toggle",
                                    name=L["ANNOUNCE_NAME"],
                                    desc=L["ANNOUNCE_DESC"],
                                    order=4,
                                    get=function() return Procodile.db.profile.announce end,
                                    set=function() Procodile.db.profile.announce = not Procodile.db.profile.announce end,
                            },
                            announceflash = {
                                    type="toggle",
                                    name=L["ANNOUNCEFLASH_NAME"],
                                    desc=L["ANNOUNCEFLASH_DESC"],
                                    order=5,
                                    get=function() return Procodile.db.profile.announceflash end,
                                    set=function() 
                                    		Procodile.db.profile.announceflash = not Procodile.db.profile.announceflash
                                    	end,
                            },                            
                            announceshake = {
                                    type="toggle",
                                    name=L["ANNOUNCESHAKE_NAME"],
                                    desc=L["ANNOUNCESHAKE_DESC"],
                                    order=6,
                                    get=function() return Procodile.db.profile.announceshake end,
                                    set=function() 
                                    		Procodile.db.profile.announceshake = not Procodile.db.profile.announceshake
                                    	end,
                            },							
                       }
                },
                                
        },
}