Procodile = LibStub("AceAddon-3.0"):NewAddon("Procodile", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0", "LibBars-1.0", "LibSink-2.0")
local L = LibStub("AceLocale-3.0"):GetLocale("Procodile", false)
local bars = LibStub("LibBars-1.0")
local media = LibStub("LibSharedMedia-3.0")
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local icon = LibStub("LibDBIcon-1.0")

local dataobj = ldb:NewDataObject("Procodile", {type = "data source", icon = "Interface\\Icons\\Spell_Holy_WordFortitude", text = ""})

BINDING_HEADER_Procodile = "Procodile"
BINDING_NAME_TOGGLE_PROCODILE = "Toggle"

local changed = true
local cooldowns
local auras

function Procodile:OnInitialize()
	self:RegisterChatCommand("procodile", "Command")
	self:RegisterChatCommand("proc", "Command")
	self:RegisterChatCommand("proctest", "test")
	self:RegisterChatCommand("procscan", "ScanForProcs")
	
	self.db = LibStub("AceDB-3.0"):New("ProcodileDB", self.defaults)
	
	self.options.args.output = Procodile:GetSinkAce3OptionsDataTable()
	self.options.args.output.disabled = false
	self:SetSinkStorage(self.db.profile.sinkOptions)
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Procodile", self.options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Procodile", "Procodile")
	
	LibStub("AceConfig-3.0"):RegisterOptionsTable("Procodile-Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db))
	self.profilesFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("Procodile-Profiles", "Profiles", "Procodile")

	self:SetupBars()

	-- Minimap button
	icon:Register("Procodile", dataobj, Procodile.db.profile.icon)
	self:ShowMMButton(self.db.profile.mmbutton)
end

function Procodile:ShowMMButton(show)
	if(show) then
		icon:Show("Procodile")
	else
		icon:Hide("Procodile")
	end
end

function Procodile:Command(input)
	self:OpenOptions()
end

function Procodile:OpenOptions()
	InterfaceOptionsFrame_OpenToCategory("Procodile")
end

function Procodile:OnEnable()
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED")
	self:RegisterEvent("UNIT_INVENTORY_CHANGED")
	changed = true
	
	-- Hook into GameTooltip, for displaying procs on items.
	GameTooltip:HookScript("OnTooltipSetItem", function(tooltip, item) Procodile:OnTooltipSetItem(tooltip,item) end)

	-- Clean up tracked spells
	for internalname,proc in pairs(self.db.profile.tracked) do
		proc.laststarted = 0
		proc.started = 0
	end
	
	-- Check for item procs
	self:ScanForProcs()
	
	-- Should we enable tracking straight away?
	if self.db.profile.combat and InCombatLockdown() == nil then
		self:Toggle(false)
	else
		self:Toggle(true)
	end
	
	self:ScheduleRepeatingTimer("Tick", 1, nil)
	
end

function Procodile:OnDisable()
	self:CancelAllTimers()
	self.db.profile.enabled = false
	Procodile:Toggle(false)
end

function Procodile:ScanForProcs()
	-- Mark all tracked spells as not found after this inventory change
	for index, spell in ipairs(self.db.profile.tracked) do
		spell.found = false
		
		-- Custom spells should never be automatically removed 
		if spell.custom ~= nil then
			spell.found = true
		end
	end
	
	-- Talents
	-- TODO: don't scan every time
	local myclass = select(2, UnitClass("player"))
	for spell_id,theclass in pairs(self.talents) do
		if theclass == myclass then
			
			-- See if we are already tracking this item
			local exists = false
			for index, spell in pairs(self.db.profile.tracked) do
				if spell.id == spell_id then
					exists = true
					spell.found = true
				end
			end
			
			-- No, add it
			if not exists then
				local name, rank, icon, cost, isFunnel, powerType, castTime, minRange, maxRange = GetSpellInfo(spell_id)
				self:AddSpell(spell_id, name, icon)
			end
		end
	end

	-- For each inventory slot (ugly, should refer by IDs given by GetInventorySlotInfo()).
	for slotid = 1, 20 do
	
		local itemlink = GetInventoryItemLink("player", slotid)
		if itemlink ~= nil then
		
			-- Thank you, wowwiki
			local found, _, itemstring = string.find(itemlink, "^|c%x+|H(.+)|h%[.+%]")
			local _, itemId, enchantId, jewelId1, jewelId2, jewelId3, jewelId4, suffixId, uniqueId = strsplit(":", itemstring)
			local itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture = GetItemInfo(itemlink)
			
			-- Item enchants
			if tonumber(enchantId) ~= 0 then
			
				for spell_id,spell_enchantid in pairs(self.enchants) do
					if spell_enchantid == tonumber(enchantId) then
						
						-- See if we are already tracking this item
						local exists = false
						for index, spell in pairs(self.db.profile.tracked) do
							if spell.id == spell_id then
								exists = true
								spell.found = true
							end
						end
						
						-- No, add it
						if not exists then
							self:AddSpell(spell_id, itemName, itemTexture)
						end
						
						break
					end
				end
				
			end

			-- Item proc spells
			for spell_id,spell_itemid in pairs(self.items) do
				if spell_itemid == tonumber(itemId) then
					
					-- See if we are already tracking this item
					local exists = false
					for index, spell in pairs(self.db.profile.tracked) do
						if spell.id == spell_id then
							exists = true
							spell.found = true
						end
					end
					
					-- No, add it
					if not exists then
						local spell = self:AddSpell(spell_id, itemName, itemTexture)
						spell.itemId = spell_itemid
					end
					
					break
				end
			end
		end
		
	end
	
	-- Remove spells not found (item removed from inventory)
	for index, spell in pairs(self.db.profile.tracked) do
		if not spell.found then
			self:RemoveSpell(spell.id)
		end
	end
		
end

function Procodile:ShowCooldowns(show)
	if show then
		self.cooldowns:Show()
	else
		self.cooldowns:Hide()
	end
end

function Procodile:ShowAurabars(show)
	if show then
		self.auras:Show()
	else
		self.auras:Hide()
	end
end

function Procodile:ReverseCooldowns(reverse)
	self.cooldowns:ReverseGrowth(reverse)
end

function Procodile:ReverseAurabars(reverse)
	self.auras:ReverseGrowth(reverse)
end

-- Returns all currently tracked spells
function Procodile:GetTrackedSpells()
	return self.db.profile.tracked
end

-- Adds a spell to the tracked spells
function Procodile:AddSpell(spellid, itemname, itemicon, iscustom)
	if self.db.profile.verbose then
		self:Print("adding "..itemname)
	end

	-- Attempt to find the spell in dormant
	local found = false
	for index, spell in pairs(self.db.profile.dormant) do
		if spell.id == spellid then
			found = true
			
			spell.started = 0
			spell.laststarted = 0
			spell.found = true
			table.insert(self.db.profile.tracked, spell)
			table.remove(self.db.profile.dormant, index)
			
			return spell
		end
	end
	
	if not found then
		-- Spell not found in dormant - create new
		local spell = {
					id = spellid, 
					name = itemname, 
					icon = itemicon, 
					seconds = 0, 
					count = 0, 
					started = 0, 
					totaltime = 0, 
					cooldown = 0, 
					laststarted = 0,
					found = true,
					custom = iscustom,
				}
		table.insert(self.db.profile.tracked, spell)
		
		return spell
	end
end

function Procodile:RemoveSpell(spellid)
	for index, spell in pairs(self.db.profile.tracked) do
		if spell.id == spellid then
			-- Add to dormant
			table.insert(self.db.profile.dormant, spell)
			
			-- Remove from tracked
			table.remove(self.db.profile.tracked, index)
			
			self.cooldowns:RemoveBar(spell.name)
			
			if self.db.profile.verbose then
				self:Print("removing "..spell.name)
			end
			return
		end
	end
end

function Procodile:Reset()
	totaltime = 0
	for key,proc in pairs(self.db.profile.tracked) do
		proc.count = 0
		proc.seconds = 0
		proc.started = 0
		proc.totaltime = 0
		proc.laststarted = 0
		proc.cooldown = 0
	end
	
	self.db.profile.dormant = {}
	
	self:Print("|cff22ff22".."Data for all tracked procs has been reset")
	
	ProcodileFu:OnUpdateFuBarText()
end

-- Applies settings to cooldown and aura bars
function Procodile:ReconfigureBars()
	self.cooldowns:SetFont(media:Fetch('font', self.db.profile.barfont), self.db.profile.barfontsize)
	self.cooldowns:SetWidth(self.db.profile.barwidth)
	self.cooldowns:SetHeight(self.db.profile.barheight)
	self.cooldowns:SetTexture(media:Fetch('statusbar', self.db.profile.bartexture))

	self.auras:SetFont(media:Fetch('font', self.db.profile.aurabarfont), self.db.profile.aurabarfontsize)
	self.auras:SetWidth(self.db.profile.aurabarwidth)
	self.auras:SetHeight(self.db.profile.aurabarheight)
	self.auras:SetTexture(media:Fetch('statusbar', self.db.profile.aurabartexture))
end

function Procodile:SetupBars()
	-- Cooldowns
	self.cooldowns = self:NewBarGroup("Cooldowns", nil, self.db.profile.barwidth, self.db.profile.barheight, "ProcodileCooldowns")
	self.cooldowns:SetFont(media:Fetch('font', self.db.profile.barfont), self.db.profile.barfontsize)
	self.cooldowns:SetTexture(media:Fetch('statusbar', self.db.profile.bartexture))
	self.cooldowns:SetColorAt(1.00, 1.0, 0.2, 0.2, 0.8)
	self.cooldowns:SetColorAt(0.66, 1.0, 0.3, 0.3, 0.8)
	self.cooldowns:SetColorAt(0.33, 1.0, 0.4, 0.4, 0.8)
	self.cooldowns:SetColorAt(0.00, 1.0, 0.5, 0.5, 0.8)
	self.cooldowns.RegisterCallback(self, "AnchorClicked")
	self.cooldowns.RegisterCallback(self,"TimerFinished")
	self.cooldowns:SetUserPlaced(true)
	self.cooldowns:ReverseGrowth(self.db.profile.reversecooldowns or false)

	-- Auras
	self.auras = self:NewBarGroup("Auras", nil, self.db.profile.aurabarwidth, self.db.profile.aurabarheight, "ProcodileAuras")
	self.auras:SetFont(media:Fetch('font', self.db.profile.aurabarfont), self.db.profile.aurabarfontsize)
	self.auras:SetTexture(media:Fetch('statusbar', self.db.profile.aurabartexture))
	self.auras:SetColorAt(1.00, 1.0, 0.2, 0.2, 0.8)
	self.auras:SetColorAt(0.66, 1.0, 0.3, 0.3, 0.8)
	self.auras:SetColorAt(0.33, 1.0, 0.4, 0.4, 0.8)
	self.auras:SetColorAt(0.00, 1.0, 0.5, 0.5, 0.8)
	self.auras.RegisterCallback(self, "AnchorClicked")
	self.auras.RegisterCallback(self,"TimerFinished")
	self.auras:SetUserPlaced(true)
	self.auras:ReverseGrowth(self.db.profile.reversaurabars or false)

	-- Cheat for getting anchor to work straight off; not sure why it doesn't until first bar is added.
	-- Fix at some point.
	local bar = self.cooldowns:NewTimerBar("Init bar", "Test bar", 10, 10, "Interface\\Icons\\Spell_Holy_WordFortitude")
	bar:SetHeight(self.db.profile.barheight)
	bar:SetTimer(0, 0)

	local bar = self.auras:NewTimerBar("Init bar", "Test bar", 10, 10, "Interface\\Icons\\Spell_Holy_WordFortitude")
	bar:SetHeight(self.db.profile.aurabarheight)
	bar:SetTimer(0, 0)
	
	self:ShowCooldowns(self.db.profile.cooldowns)
	self:SetMovableCooldowns(self.db.profile.movablecooldowns)
	
	self:ShowAurabars(self.db.profile.aurabars)
	self:SetMovableAurabars(self.db.profile.movableaurabars)
	
end

function Procodile:TimerFinished(evt, group, bar)
	-- Announce when a cooldown is over
	if group == self.cooldowns and self.db.profile.announce and bar.name ~= "Init bar" then -- Can you say "ugly"?
		self:Pour(bar.name.." CD expired!", 1.0, 0.5, 0.5)
		if self.db.profile.announceflash then
			self:Flash()
		end
		if self.db.profile.announceshake then
			self:Shake()
		end
	end
end

function Procodile:AnchorClicked(cbk, group, button)
	if button == "RightButton" then
		if group == self.cooldowns then
			self:SetMovableCooldowns(false)
		elseif group == self.auras then
			self:SetMovableAurabars(false)
		end
	end
end

function Procodile:SetMovableCooldowns(movable)
	self.db.profile.movablecooldowns = movable
	if movable then
		self.cooldowns:ShowAnchor()
	else
		self.cooldowns:HideAnchor()
	end
end

function Procodile:SetMovableAurabars(movable)
	self.db.profile.movableaurabars = movable
	if movable then
		self.auras:ShowAnchor()
	else
		self.auras:HideAnchor()
	end
end

function Procodile:Tick()
	if self.db.profile.enabled then
		for key,proc in pairs(self.db.profile.tracked) do
			if proc.started > 0 then
				proc.seconds = proc.seconds + 1
			end
			proc.totaltime = proc.totaltime + 1
		end
	end
end

function Procodile:TestBars()
	local bar = self.cooldowns:NewTimerBar("Test bar", "Test bar", 10, 10, "Interface\\Icons\\Spell_Holy_WordFortitude")
	bar:SetHeight(self.db.profile.barheight)
	bar:SetTimer(10, 10)
	
	local anotherbar = self.cooldowns:NewTimerBar("Test bar 2", "Test bar", 10, 10, "Interface\\Icons\\Spell_Holy_WordFortitude")
	anotherbar:SetHeight(self.db.profile.barheight)
	anotherbar:SetTimer(5, 5)

	local aura = self.auras:NewTimerBar("Test bar", "Test bar", 10, 10, "Interface\\Icons\\Spell_Holy_WordFortitude")
	bar:SetHeight(self.db.profile.aurabarheight)
	bar:SetTimer(10, 10)

	local anotheraura = self.auras:NewTimerBar("Test bar 2", "Test bar", 10, 10, "Interface\\Icons\\Spell_Holy_WordFortitude")
	anotheraura:SetHeight(self.db.profile.aurabarheight)
	anotheraura:SetTimer(5, 5)
end

function Procodile:test()
	for key,proc in pairs(self.db.profile.tracked) do
		self:Print("key="..key)
		
		self:Print("id="..proc.id)
		self:Print("count="..proc.count)
		self:Print("seconds="..proc.seconds)
		self:Print("started="..proc.started)
	end
end

function Procodile:PLAYER_REGEN_ENABLED()
	-- Disable if we are enabled and are set to only track in combat
	if self.db.profile.enabled and self.db.profile.combat then
		Procodile:Toggle(false)
	end
end

function Procodile:PLAYER_REGEN_DISABLED()
	-- Enable if we are not enabled and are set to only track in combat
	if not self.db.profile.enabled and self.db.profile.combat then
		Procodile:Toggle(true)
	end
end

function Procodile:UNIT_INVENTORY_CHANGED(event, unit)
	if unit == "player" then
		self:ScanForProcs()
	end
end

function Procodile:HandleProc(proc, is_aura, duration)
	if proc.laststarted > 0 and (proc.cooldown == 0 or (time() - proc.laststarted < proc.cooldown)) then
		proc.cooldown = time() - proc.laststarted
	
		if self.db.profile.announce and proc.cooldown < 300 then
			self:Pour(proc.name.." new cooldown detected: "..proc.cooldown.."s", 1.0, 0.5, 0.5)
		end
	end
	
	if is_aura then
		proc.started = time()
	end
	proc.count = proc.count + 1
	proc.laststarted = time()
	if self.db.profile.verbose then
		self:Print("|cff22ff22"..proc.name.." effect")
	end
	
	-- Reset cooldown bar
	if self.db.profile.cooldowns and proc.cooldown > 0 then
		local bar = self.cooldowns:GetBar(proc.name)
		if not bar then
			bar = self.cooldowns:NewTimerBar(proc.name, proc.name, proc.cooldown, proc.cooldown, proc.icon)
		end
		bar:SetHeight(self.db.profile.barheight)
		bar:SetTimer(proc.cooldown, proc.cooldown)
	end
	
	-- Aura bars
	if is_aura and duration > 0 and self.db.profile.aurabars then
		-- TODO: show target if this is aura is not on self (test with Hunter's Mark)
		local bar = self.auras:GetBar(proc.name)
		if not bar then
			bar = self.auras:NewTimerBar(proc.name, proc.name, durartion, duration, proc.icon)
		end
		bar:SetHeight(self.db.profile.barheight)
		bar:SetTimer(duration, duration)

		if self.db.profile.announce then
			self:Pour(proc.name.."!", 1.0, 0.5, 0.5)
		end
		if self.db.profile.announceflash then
			self:Flash()
		end
		if self.db.profile.announceshake then
			self:Shake()
		end
		
	end
end

function Procodile:COMBAT_LOG_EVENT_UNFILTERED(event, timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
		local obj = select(1, ...)
		local match = false
      	
       if self.db.profile.enabled then

	        if dstName ~= nil and srcName ~= nil and UnitIsUnit(srcName,"player") then

				-- Instant dmg by you on target
				if eventtype == 'SPELL_DAMAGE' and srcName ~= nil and UnitIsUnit(srcName,"player") then
--12/2 19:44:04.312  SPELL_DAMAGE,0x00000000004518D2,"Zo",0x512,0xF13000402D0060F6,"Soul Weaver",0x100a48,60488,"Extract of Necromatic Power",0x20,2065,0,32,0,0,0,1,nil,nil
--12/2 19:50:46.765  SPELL_DAMAGE,0x0000000000426DC0,"Ladybah",0x512,0xF13000402B006472,"Soldier of the Frozen Wastes",0xa48,55637,"Lightweave Bolt",0x2,1955,293,2,0,0,0,1,nil,nil

		        	for internalname,proc in pairs(self.db.profile.tracked) do
		        		if proc.id == obj then
		        			match = true
		        		
		        			self:HandleProc(proc, false)
			        		
			        		changed = true
		        		end
		        	end

		        	if not match and self.db.profile.showspells then
				       	local spellname = select(2, ...)
			        	self:Print("Spell "..spellname.." ("..obj..") hit enemy.")
		        	end					

				end

				-- Instant energize on self/target
				if eventtype == 'SPELL_ENERGIZE' and srcName ~= nil and UnitIsUnit(srcName,"player") then
--12/2 19:43:19.578  SPELL_ENERGIZE,0x000000000035F51C,"Dimqroy",0x514,0x000000000035F51C,"Dimqroy",0x514,60538,"Soul of the Dead",0x40,900,0
		        	for internalname,proc in pairs(self.db.profile.tracked) do
		        		if proc.id == obj then
		        			match = true
		        			
		        			self:HandleProc(proc, false)
			        		
			        		changed = true
		        		end
		        	end

		        	if not match and self.db.profile.showspells then
				       	local spellname = select(2, ...)
			        	self:Print("Spell "..spellname.." ("..obj..") energized.")
		        	end					
				end
	
	        	-- Aura applied on self/target
--12/2 18:53:03.937  SPELL_AURA_APPLIED,0xF14018E46D000022,"Sturm",0x1114,0xF14018E46D000022,"Sturm",0x1114,34456,"Ferocious Inspiration",0x1,BUFF
--event, timestamp, eventtype, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...
		        if eventtype == 'SPELL_AURA_APPLIED' and srcName ~= nil and UnitIsUnit(srcName,"player") then

			      	--self:Print(obj.." aura applied, name "..select(2, ...))

		        	for internalname,proc in pairs(self.db.profile.tracked) do
		        		if proc.id == obj then
		        			match = true
		        			
		        			if self.db.profile.aurabars then
					        	local duration = self:GetAuraDuration(dstName,select(2,...))
								self:HandleProc(proc, true, duration)
			        		else
								self:HandleProc(proc, true, 0)
							end
			        		
			        		changed = true
		        		end
		        	end

		        	if not match and self.db.profile.showspells then
				       	local spellname = select(2, ...)
			        	self:Print("Spell "..spellname.." ("..obj..") applied.")
		        	end
		        end
		
				-- Refresh on an aura; we are only interested because we may need to update an aura bar.
				-- TODO: maybe treat this as an aura application, since a proc is a proc, and we won't get a SPELL_AURA_APPLIED the second time.
				if eventtype == 'SPELL_AURA_REFRESH' then
				
				end
		
		        -- Aura removed on self/target
		        if eventtype == 'SPELL_AURA_REMOVED' and srcName ~= nil and UnitIsUnit(srcName,"player") then
		        	for internalname,proc in pairs(self.db.profile.tracked) do
		        		if proc.id == obj then
		        			match = true
		        		
			        		proc.started = 0
			        		if self.db.profile.verbose then
				        		self:Print("|cff22ff22"..proc.name.." effect ended")
			        		end
			        		
			        		changed = true
		        		end
		        	end
		        	
		        	if not match and self.db.profile.showspells then
				       	local spellname = select(2, ...)
			        	self:Print("Spell "..spellname.." ("..obj..") removed.")
		        	end
		        end
				
--	        	self:Print("|cff22ff22"..eventtype..', srcGUID='..srcGUID..', dstGUID='..dstGUID..', obj='..obj)
	        end
       	
       end
end

function Procodile:GetAuraDuration(unit, name)

	if UnitIsUnit(unit,"player") then
		--self:Print(name.." is self")
	
		for i=1,40 do
			local bname, brank, bicon, bcount, bdebuffType, bduration, bexpirationTime, bisMine, bisStealable = UnitBuff(unit, i)
			if bname == nil then
				break
			end
			if bname == name and bisMine then
				--self:Print("buff on "..unit..": "..bname..", duration: "..bduration)
				return bduration
			end
		end

		for i=1,40 do
			local bname, brank, bicon, bcount, bdebuffType, bduration, bexpirationTime, bisMine, bisStealable = UnitDebuff(unit, i)
			if bname == name and bisMine then
				--self:Print("debuff on "..unit..": "..bname..", duration: "..bduration)
				return bduration
			end
		end

	else 
		--self:Print(name.." is not self")

		for i=1,40 do
			local bname, brank, bicon, bcount, bdebuffType, bduration, bexpirationTime, bisMine, bisStealable = UnitDebuff(unit, i)
			if bname ~= nil then
				--self:Print("debuff on "..unit..": "..bname)
				if bname == name and bisMine then
					--self:Print("duration: "..bduration)
					return bduration
				end
			end
		end
		
		for i=1,40 do
			local bname, brank, bicon, bcount, bdebuffType, bduration, bexpirationTime, bisMine, bisStealable = UnitBuff(unit, i)
			if bname ~= nil then
				--self:Print("buff on "..unit..": "..bname)
				if bname == name and bisMine then
					--self:Print("duration: "..bduration)
					return bduration
				end
			end
		end		
	end

	return 0
end

-- Shamelessly copied from Omen - thanks!
function Procodile:Flash()
	if not self.FlashFrame then
		local flasher = CreateFrame("Frame", "ProcodileFlashFrame")
		flasher:SetToplevel(true)
		flasher:SetFrameStrata("FULLSCREEN_DIALOG")
		flasher:SetAllPoints(UIParent)
		flasher:EnableMouse(false)
		flasher:Hide()
		flasher.texture = flasher:CreateTexture(nil, "BACKGROUND")
		flasher.texture:SetTexture("Interface\\FullScreenTextures\\LowHealth")
		flasher.texture:SetAllPoints(UIParent)
		flasher.texture:SetBlendMode("ADD")
		flasher:SetScript("OnShow", function(self)
			self.elapsed = 0
			self:SetAlpha(0)
		end)
		flasher:SetScript("OnUpdate", function(self, elapsed)
			elapsed = self.elapsed + elapsed
			if elapsed < 2.6 then
				local alpha = elapsed % 1.3
				if alpha < 0.15 then
					self:SetAlpha(alpha / 0.15)
				elseif alpha < 0.9 then
					self:SetAlpha(1 - (alpha - 0.15) / 0.6)
				else
					self:SetAlpha(0)
				end
			else
				self:Hide()
			end
			self.elapsed = elapsed
		end)
		self.FlashFrame = flasher
	end

	self.FlashFrame:Show()
end

-- Shamelessly copied from Omen (which copied from BigWigs) - thanks!
function Procodile:Shake()
	local shaker = self.ShakerFrame
	if not shaker then
		shaker = CreateFrame("Frame", "ProcodileShaker", UIParent)
		shaker:Hide()
		shaker:SetScript("OnUpdate", function(self, elapsed)
			elapsed = self.elapsed + elapsed
			local x, y = 0, 0 -- Resets to original position if we're supposed to stop.
			if elapsed >= 0.8 then
				self:Hide()
			else
				x, y = random(-8, 8), random(-8, 8)
			end
			if WorldFrame:IsProtected() and InCombatLockdown() then
				if not shaker.fail then
					Omen:Print(L["|cffff0000Error:|r Procodile cannot use shake warning if you have turned on nameplates at least once since logging in."])
					shaker.fail = true
				end
				self:Hide()
			else
				WorldFrame:ClearAllPoints()
				for i = 1, #self.originalPoints do
					local v = self.originalPoints[i]
					WorldFrame:SetPoint(v[1], v[2], v[3], v[4] + x, v[5] + y)
				end
			end
			self.elapsed = elapsed
		end)
		shaker:SetScript("OnShow", function(self)
			-- Store old worldframe positions, we need them all, people have frame modifiers for it
			if not self.originalPoints then
				self.originalPoints = {}
				for i = 1, WorldFrame:GetNumPoints() do
					tinsert(self.originalPoints, {WorldFrame:GetPoint(i)})
				end
			end
			self.elapsed = 0
		end)
		self.ShakerFrame = shaker
	end

	shaker:Show()
end

function Procodile:IsEnabled()
	return self.db.profile.enabled
end

function Procodile:Toggle(enabled)
	if enabled ~= nil then
		self.db.profile.enabled = enabled
	else
        self.db.profile.enabled = not self.db.profile.enabled
	end

	if self.db.profile.verbose then
		if self.db.profile.enabled then
			self:Print("|cff22ff22"..L["TOGGLE_ENABLED"])
		else
			self:Print("|cffff2222"..L["TOGGLE_DISABLED"])
		end
	end
	
	-- Reset some variables we only track while enabled
	if not self.db.profile.enabled then
       	for internalname,proc in pairs(self.db.profile.tracked) do
       		proc.laststarted = 0
       		proc.started = 0
       	end
	end
	
	self:UpdateText(self.db.profile.enabled)
end

function Procodile:PopulateTooltip()

    GameTooltip:AddLine(L["Procodile"])
    GameTooltip:AddLine(" ")
    
    local tracked = Procodile:GetTrackedSpells()
    
	for internalname,proc in pairs(tracked) do
		GameTooltip:AddLine(proc.name, 0.3, 0.3, 0.8)
		GameTooltip:AddTexture(proc.icon)
		self:PopulateTooltipWithProc(proc)
		GameTooltip:AddLine(" ")
	end
end

function Procodile:PopulateTooltipWithProc(proc)
	-- PPM
	local ppm = 0
	if proc.count > 0 then
		ppm = proc.count / (proc.totaltime / 60)
	end
	
	-- Uptime
	local uptime = 0
	if proc.seconds > 0 and proc.totaltime > 0 then
		uptime = proc.seconds / proc.totaltime * 100
	end
			
	if proc.count > 0 then
	    GameTooltip:AddDoubleLine(L["Procs"], proc.count, nil, nil, nil, 1,1,1)
	else
	    GameTooltip:AddDoubleLine(L["Procs"], "none", nil, nil, nil, 1,1,1)
	end
		
    if ppm > 0 then
	    GameTooltip:AddDoubleLine(L["PPM"], string.format("%.2f", ppm), nil, nil, nil, 1,1,1)
	else
	    GameTooltip:AddDoubleLine(L["PPM"],"unknown", nil, nil, nil, 1,1,1)
	end
	
	if uptime > 0 then
	    GameTooltip:AddDoubleLine(L["Uptime"], string.format("%.2f", uptime).."%", nil, nil, nil, 1,1,1)
	else
	    GameTooltip:AddDoubleLine(L["Uptime"], "unknown", nil, nil, nil, 1,1,1)
	end
	
    if proc.cooldown > 0 then
	    GameTooltip:AddDoubleLine(L["Cooldown"], proc.cooldown.."s", nil, nil, nil, 1,1,1)
	else
	    GameTooltip:AddDoubleLine(L["Cooldown"], "unknown", nil, nil, nil, 1,1,1)
	end
end

function Procodile:UpdateText(enabled)
	if enabled then
		dataobj.text = "Enabled"
	else
		dataobj.text = "Disabled"
	end
end

function dataobj:OnEnter()
    GameTooltip:SetOwner(self, "ANCHOR_NONE")
    GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
    GameTooltip:ClearLines()

    Procodile:PopulateTooltip()
    
    GameTooltip:AddLine(L["Hint: Ctrl-Click to reset."], 0, 1, 0)
    GameTooltip:AddLine(L["Shift-click to toggle tracking."], 0, 1, 0)
    GameTooltip:AddLine(L["Right-click to configure"], 0, 1, 0)
    
    GameTooltip:Show()
end

function dataobj:OnLeave()
    GameTooltip:Hide()
end

function dataobj:OnClick(button)
	if button == "LeftButton" and IsShiftKeyDown() then
		Procodile:Toggle()
	elseif button == "LeftButton" and IsControlKeyDown() then
		Procodile:Reset()
	elseif button == "RightButton" then
		Procodile:OpenOptions()
	end
end

function Procodile:OnTooltipSetItem(tooltip, ...)
	local itemLink = select(2, tooltip:GetItem())
	local found, _, itemString = string.find(itemLink, "^|c%x+|H(.+)|h%[.*%]")
	local itemId = select(2, strsplit(":", itemString))
	
	-- Tracked items
	for index, spell in pairs(self.db.profile.tracked) do
		if spell.itemId ~= nil and tostring(spell.itemId) == itemId then
			self:PopulateTooltipWithProc(spell)
		end
	end
	
	-- Dormant items
	for index, spell in pairs(self.db.profile.dormant) do
		if spell.itemId ~= nil and tostring(spell.itemId) == itemId then
			self:PopulateTooltipWithProc(spell)
		end
	end
end
